/*
 * Copyright 2010 Tilburg University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package nl.uvt.pdproxy;

import java.io.*;
import java.util.*;
import org.apache.log4j.*;

public class Configuration {
  public static String EXPERTS_AND_EXPERTISE_URL_TEMPLATE = "pdproxy.experts-and-expertise.url-template";
  public static String CACHE_DIRECTORY = "pdproxy.file-system-cache.directory";
  public static String CACHE_MAX_AGE = "pdproxy.cache.max-age";
  public static String STORE_DIRECTORY = "pdproxy.file-system-store.directory";
  public static String STORE_MIN_AGE = "pdproxy.store.min-age";
  
  private String configFileName;
  private Properties properties;
  private List<File> configDirectories;

  public Configuration(List<File> configDirectories) {
    this.configDirectories = configDirectories;
    this.configFileName = "pdproxy.properties";
    this.properties = new Properties();
  }

  public void setConfigFileName(String configFileName) {
    this.configFileName = configFileName;
  }

  /**
   * Loads properties from directories list or classpath.
   * 
   * <p>It first tries to load the properties from the list of directories, then from
   * the classpath. If a log4j.properties file is included in a Jar or War file, the latter
   * method will load it.</p>
   */
  public void loadProperties() {
    try {
      InputStream is = null;
      File file = this.getFile(this.configFileName);
      if (file != null) {
        is = new FileInputStream(file);
      }
      else {
        is = this.getClass().getClassLoader().getResourceAsStream(this.configFileName);
      }
      this.properties.load(is);
    } catch (FileNotFoundException e) {
      System.err.println("FATAL - Config file not found");
    } catch (IOException e) {
      System.err.println("ERROR - IOException while reading config file, message="+e.getMessage());
    }
  }
  
  public File getFile(String name) {
    File file = null;
    boolean foundit = false;
    for (int i=0; foundit == false && i<this.configDirectories.size(); i++) {
      File directory = this.configDirectories.get(i);
      file = new File(directory, name);
      if (file.exists()) {
        System.out.println("INFO - Using "+file.getAbsolutePath());
        foundit = true;
      }
    }
    if (foundit == true)
      return file;
    else
      return null;
  }

  public String getProperty(String property) {
    String retval = null;
    retval = this.properties.getProperty(property);
    return retval;
  }
  
  public void setupLog4JConfiguration() {
    File log4jPropertiesFile = this.getFile("log4j.properties");
    if (log4jPropertiesFile != null) {
      PropertyConfigurator.configureAndWatch(log4jPropertiesFile.getAbsolutePath());
    }
    else {
      BasicConfigurator.configure();
    }
  }  

}
